# -*- coding: utf-8 -*-
import re

questions_fn = "./data/questions-t1.txt"
results_filename = "./t1_results.txt"

test_questions_fn = "./data/test-questions-t1.txt"
test_results_filename = "./test-t1_results.txt"

# Mettre dans cette partie les expressions régulières
# que vous utilisez pour analyser les questions
# et en faire la conversion en affirmations
#
# Vos regex...
#

def run_task1(filename):
    questions = load_questions(filename)
    results = convert_all_questions(questions)
    save_results(results)

def run_task_test(filename, test_results_filename):
    questions = load_questions(filename)
    results = convert_all_questions(questions)
    save_results(results, test_results_filename)


def convert_all_questions(questions):
    results = []
    for question in questions:
        sentence = convert(question)
        results.append((question, sentence))
    return results


def load_questions(filename):
    with open(filename, 'r', encoding='utf-8') as f:
        raw_questions = f.readlines()
    questions = [x.strip() for x in raw_questions]
    return questions


def convert(question):
    # Insérer ici votre code pour le traitement des questions et leur conversion.
    # Vous pouvez ajouter autant de fonctions que vous souhaitez.
    # Voir fichier data/results-reference-t1.txt pour des exemples de conversion.
    #
    # IMPORTANT : Ne pas modifier les autres fonctions existantes de ce fichier
    #             afin de faciliter mon travail de correction.
    #             En cas de doute, me consulter.
    # Votre code...

    regex_qui = re.compile(r"(qui)\s"
                           r"(est|sont|était|étaient|sera|seront)\s"
                           r"(.*)\?",
                           re.IGNORECASE)
    modif1_0 = re.sub(regex_qui,
                    r"\3 \2 Luc Lamontagne.",
                    question)

    regex_qui_a = re.compile(r"(qui)\s"
                           r"(a)\s"
                           r"(.*)\?",
                           re.IGNORECASE)
    modif1 = re.sub(regex_qui_a,
                    r"Luc Lamontagne \2 \3.",
                    modif1_0)

    regex_quel_nom = re.compile(r"quel\s"
                                r"(est)\s"
                                r"(le nom du chef)"
                                r"(.*)\?",
                                re.IGNORECASE)
    modif1_1 = re.sub(regex_quel_nom,
                      r"\2 \3 \1 Luc Lamontagne.",
                      modif1)

    regex_questce = re.compile(r"(qu'est-ce\squ[ie'])\s?"
                               r"(.*)\?",
                               re.IGNORECASE)
    modif2 = re.sub(regex_questce,
                    r"\2 est X.",
                    modif1_1)

    regex_quesont = re.compile(r"(qu[e'])\s"
                               r"(est|sont|signifie|fait)-?(on)?"
                               r"(.*)\?",
                               re.IGNORECASE)
    modif3 = re.sub(regex_quesont,
                    r"\4 \3 \2 X.",
                    modif2)

    regex_ou = re.compile(r"((dans\squel(le)?\s(ville|région|pays))|où)\s"
                          r"(est|sont)\s"
                          r"(.*)\?",
                          re.IGNORECASE)

    modif4 = re.sub(regex_ou,
                    r"\6 \5 à Québec.", modif3)

    regex_capitale = re.compile(r"(quel[le]*)\s(est)\s(la\scapitale|l'emplacement)(.*)\?", re.IGNORECASE)

    modif4_1 = re.sub(regex_capitale,
           r"\3 \4 \2 Québec.", modif4)

    regex_mesure = re.compile(r"([aà]\s)?qu[el]+\s(âge\s)?"
                              r"(est|avait)?\s?"
                              r"(l[a']\s?)?"
                              r"(?(2)|(hauteur|température|vitesse|distance|profondeur|espérance\sde\svie))\s"
                              r"(se)?(?(6)(\s\w+)|)(.*)\?",
                              re.IGNORECASE)
    modif5 = re.sub(regex_mesure,
                    r"\4 \5 \8 \6 \7 \3 1000.",
                    modif4_1)

    regex_quelle_est = re.compile(r"quel[les]*\s(.*)(est|sont)\s(.*)\?",
                                  re.IGNORECASE)
    modif6 = re.sub(regex_quelle_est, r"\1\3 \2 X.", modif5)

    regex_quand = re.compile(r"(quand|en\squelle\sannée)\s(est\sné\s)?(.*)\?",
                             re.IGNORECASE)
    modif7 = re.sub(regex_quand,
                    r"\3 \2en 2000.",
                    modif6)

    regex_pourquoi = re.compile(r"pourquoi\s(.*)\?",
                                re.IGNORECASE)
    modif8 = re.sub(regex_pourquoi,
                    r"\1 parce que X.",
                    modif7)

    regex_combien1 = re.compile(r"combien\sde\s(\w+)(.*)\?",
                                re.IGNORECASE)
    modif9 = re.sub(regex_combien1,
                    r"\2 1000 \1.",
                    modif8)

    regex_combien2 = re.compile(r"combien\s(y\sa-t-il)?\s?de(.*)\?",
                                re.IGNORECASE)
    modif10 = re.sub(r"\by\sa-t-il",
                     r"il y a",
                     re.sub(regex_combien2,
                            r"\1 1000 \2.",
                            modif9))

    regex_quelle_fin = re.compile(r"(.*?)(qu\w+)\s(\w+)\?",
                                  re.IGNORECASE)
    modif11 = re.sub(regex_quelle_fin,
                     r"\1 \3 X.",
                     modif10)

    # Modifications non liées aux mots
    modif12 = re.sub(r"(\w+)(-t)?-(il|elle)", r"\1", modif11)
    modif12 = re.sub(r"(\w+)(-t)?-(vous)", r"\3 \1", modif12)
    modif12 = re.sub(r"\s{2,}", " ", modif12)
    modif12 = re.sub(r"^\s", "", modif12)
    modif12 = re.sub(r"\'\s", "'", modif12)

    if question != modif11:
        return modif12
    else:
        return "la terre est ronde."


def save_results(results, results_filename=results_filename):
    with open(results_filename, 'w') as output_file:
        for question, sentence in results:
            output_file.write("Q: " + question + "\n")
            output_file.write("A: " + sentence + "\n")
            output_file.write("\n")


if __name__ == '__main__':
    # Vous pouvez modifier cette section
    print("Conversion des questions du fichier {} ".format(questions_fn))
    run_task1(questions_fn)
    run_task_test(test_questions_fn, test_results_filename)
    print("Conversion complétée")
